function zi=interp2(varargin)
    [nargout, nargin]=argn(0);
    bypass = 0;
uniform = 1;

if nargin>1 then
    if type(varargin($))==10 then
        narg=nargin-1;
        method=varargin($)+'    ';
        ExtrapVal=%nan;
        index=1;
    elseif (type(varargin($-1))==10 & type(varargin($))==[1 5 8])
        narg = nargin-2;
        method =  varargin($-1) + '    ';
        ExtrapVal = varargin($); 
        index = 2; 
    else
        narg = nargin;
        method = 'linear';
        ExtrapVal = %nan;
    end
    if method(1)=='*' then // Direct call bypass.
     narg = nargin-1;
     if (narg ==5 | narg ==3) then
         xitemp = varargin($ - 2);
         yitemp = varargin($-1);
         if isrowvector(xitemp) & iscolvector(yitemp)
             varargin($-2) = repmat(xitemp, [size(yitemp,1), 1]); 
             varargin($-1) = repmat(yitemp, [1, size(xitemp,2)]);
         elseif iscolvector(xitemp) & isrowvector(yitemp)  
             varargin($-2) = repmat(xitemp, [1, size(yitemp, 2)]); 
             varargin($-1) = repmat(yitemp, [size(xitemp,1), 1]);
         end
     end
     if method(2)=='l' | or(method(2:4)=='bil') then
        zi = bilinear(varargin(1:$-index), ExtrapVal);
        return
     else
         error('interp2:InvalidMethod');
     end

  end
else
  narg = nargin;
  method = 'linear';
  ExtrapVal =%nan; 
end
if narg==1 then // interp2(z), Expand Z
  [nrows,ncols] = size(varargin(1));
  xi = 1:.5:ncols; yi = (1:.5:nrows)';
  x = 1:ncols; y = (1:nrows);
  [msg,x,y,z,xi,yi] = xyzchk(x,y,varargin(1),xi,yi);
elseif narg==2 // interp2(z,n), Expand Z n times
  [nrows,ncols] = size(varargin(1));
  varg=varargin(2);
  ntimes = floor(varg(1));
  xi = 1:1/(2^ntimes):ncols; yi = (1:1/(2^ntimes):nrows)';
  x = 1:ncols; y = (1:nrows);
  [msg,x,y,z,xi,yi] = xyzchk(x,y,varargin(1),xi,yi);

elseif narg==3 // interp2(z,xi,yi)
  [nrows,ncols] = size(varargin(1));
  x = 1:ncols; y = (1:nrows);
  [msg,x,y,z,xi,yi] = xyzchk(x,y,varargin(1:3));

elseif narg==4,
  error('interp2: Wrong number of input arguments.');

elseif narg==5 // linear(x,y,z,xi,yi)
  [msg,x,y,z,xi,yi] = xyzchk(varargin(1:5));
end
if(msg~='') then
error(msg);
end

xx = x(1,:); yy = y(:,1);
if (size(x,2)>1 & ~isequal(repmat(xx,size(x,1),1),x)) | (size(y,1)>1 & ~isequal(repmat(yy,1,size(y,2)),y)) then
  error('X and Y must be matrices produced by MESHGRID. Use GRIDDATA instead of INTERP2 for scattered data.');
end
if ~bypass,
  xx = xx.'; // Make sure it's a column.
  dx = diff(xx); dy = diff(yy);
  xdiff = max(abs(diff(dx)));
   if isempty(xdiff) then
        xdiff = 0;
   end
  ydiff = max(abs(diff(dy))); 
  if isempty(ydiff) then 
      ydiff = 0; 
  end
  if (xdiff > %eps*max(abs(xx))) | (ydiff > %eps*max(abs(yy))) then
    if or(dx < 0) then // Flip orientation of data so x is increasing.
      x = x(:,$:-1:1); 
      y = y(:,$:-1:1); 
      z = z(:,$:-1:1);
      xx = xx($:-1:1,:); 
      dx = -dx($:-1:1,:);
    end
    if or(dy < 0) then // Flip orientation of data so y is increasing.
      x = x($:-1:1,:); y = y($:-1:1,:); z = z($:-1:1,:);
      yy = yy($:-1:1,:); dy = -dy($:-1:1,:);
    end
  
    if or(dx<=0) | or(dy<=0) then 
      error('X and Y must be monotonic vectors or matrices produced by MESHGRID.');
    end
    if method(1)~='c', 
      // Determine the nearest location of xi in x
      [xxi,j] = mtlb_sort(xi(:));
      [dum,i] = mtlb_sort([xx;xxi]);
      ui(i) = (1:max(size(i)));
      ui = (ui(max(size(xx))+1:$)-(1:max(size(xxi))))'; 
      ui(j) = ui;
    
      // Map values in xi to index offset (ui) via linear interpolation
      ui(ui<1) = 1;
      ui(ui>max(size(xx))-1) = max(size(xx))-1;
      ui = ui + (xi(:)-xx(ui))./(xx(ui+1)-xx(ui));
     
      // Determine the nearest location of yi in y
      [yyi,j] = mtlb_sort(yi(:));
      [dum,i] = mtlb_sort([yy;yyi(:)]);
      vi(i) = (1:length(i));
      vi = (vi(length(yy)+1:$)-(1:length(yyi)))';
      vi(j) = vi;
    
      // Map values in yi to index offset (vi) via linear interpolation
      vi(vi<1) = 1;
      vi(vi>max(size(yy))-1) = max(size(yy))-1;
      vi = vi + (yi(:)-yy(vi))./(yy(vi+1)-yy(vi));
      
      [x,y] = meshgrid(ones(type(x)):size(x,2),ones(type(y)):size(y,1));
      xi(:) = ui; yi(:) = vi;
    else
      uniform = 0;
    end
  end
end

method = convstr(method,'l')+'   ';
zi = bilinear(x,y,z,xi,yi,ExtrapVal);
endfunction

function irv = isrowvector(rv)
// check if input is row vector
irv = prod(size(rv))>=1 & size(rv,1)==1;
endfunction

function icv = iscolvector(rv)
// check if input is column vector
icv = prod(size(rv))>=1 & size(rv,2)==1;
endfunction