function [xs,ys] = snake(image, xs, ys, alpha, betA, gammA, kappa, wl, we, wt, iter);
// image: This is the image data
// xs, ys: The initial snake coordinates
// alpha: Controls tension
// beta: Controls rigidity
// gamma: Step size
// kappa: Controls enegry term
// wl, we, wt: Weights for line, edge and terminal enegy components
// iterations: No. of iteration for which snake is to be moved
ln=length(xs);
xs=interp(1:0.1:ln,1:ln,xs,splin(1:ln,xs,'not_a_knot'));
ys=interp(1:0.1:ln,1:ln,ys,splin(1:ln,ys,'not_a_knot'));

//parameters
N = iter; 
smth = double(image);

// Calculating size of image
[row col] = size(image);

//Computing external forces

eline = smth; //eline is simply the image intensities

[grady,gradx] = gradient(smth);
eedge = -1 * sqrt ((gradx .* gradx + grady .* grady)); //eedge is measured by gradient in the image
//masks for taking various derivatives
m1 = [-1 1];
m2 = [-1;1];
m3 = [1 -2 1];
m4 = [1;-2;1];
m5 = [1 -1;-1 1];

cx = convol2( smth,m1,'same');
cy = convol2( smth,m2,'same');
cxx = convol2( smth,m3,'same');
cyy = convol2( smth,m4, 'same');
cxy = convol2( smth, m5,'same');

for i = 1:row
    for j= 1:col
        // eterm as defined in Kass et al
        eterm(i,j) = (cyy(i,j)*cx(i,j)*cx(i,j) -2 *cxy(i,j)*cx(i,j)*cy(i,j) + cxx(i,j)*cy(i,j)*cy(i,j))/((1+cx(i,j)*cx(i,j) + cy(i,j)*cy(i,j))^1.5);
    end
end

eext = (wl*eline + we*eedge -wt * eterm); //eext as a weighted sum of eline, eedge and eterm

[fx, fy] = gradient(eext); //computing the gradient


//initializing the snake
xs=xs';
ys=ys';
[m n] = size(xs);
[mm nn] = size(fx);
    
//populating the penta diagonal matrix
A = zeros(m,m);

b = [(2*alpha + 6 *betA) -(alpha + 4*betA) betA];
brow = zeros(1,m);
brow(1,1:3) = brow(1,1:3) + b;
brow(1,m-1:m) = brow(1,m-1:m) + [betA -(alpha + 4*betA)]; // populating a template row
for i=1:m
    A(i,:) = brow;
    
    brow = [brow($) brow(1:$-1)]; // Template row being rotated to egenrate different rows in pentadiagonal matrix
end
[L U] = lu(A + gammA .* eye(m,m));
Ainv = inv(U) * inv(L); // Computing Ainv using LU factorization
//moving the snake in each iteration

//  for snake curve --
for i=1:N;
    ssx = gammA*xs - kappa*interp2(fx,xs,ys);
    ssy = gammA*ys - kappa*interp2(fy,xs,ys);
    //calculating the new position of snake
    xs = Ainv * ssx;
    ys = Ainv * ssy;
    z=ones(size(image,1), size(image,2));
    ys1=floor(ys);
    xs1=floor(xs);
    zz(:,:,1)=image;
    for j=1:length(xs)
        zz(ys(j),xs(j),1)=255;
        z(ys(j),xs(j))=0;
    end
    zz(:,:,2)=image.*z;
    zz(:,:,3)=image.*z;
    //Displaying the snake in its new position
    imshow(uint8(zz)); 
    xpause(500)
end
endfunction