function [msg,x,y,z,out5,out6] = xyzchk(arg1,arg2,arg3,arg4,arg5)
//
[nargout,nargin]=argn(0);
if nargin<1 | nargin>5 then
    error('Wrong number of arguments!');
end
msg='';
//msg.message = 0;
//msg.identifier = 0;
//msg = msg(zeros(0,1));

out5 = []; out6 = [];
if nargin==1 then // xyzchk(z)
  z = arg1;
  if ischar(z) then
    error('Input arguments must be numeric.');
    return
  end
  [m,n] = size(z);
  [x,y] = meshgrid(1:n,1:m);
  out5 = z; // Default color matrix
  return

elseif nargin==2 // xyzchk(z,c)
  z = arg1; c = arg2;
  [m,n] = size(z);
  [x,y] = meshgrid(1:n,1:m);
  if ~isequal(size(z),size(c)) then
    msg='Matrix C must be the same size as Z.';
    return
  end
  out5 = c;
  return

elseif nargin>=3 // xyzchk(x,y,z,...)
  x = arg1; y = arg2; z = arg3;
  [m,n] = size(z);

  if ~isvector1(z) then // z is a matrix
    // Convert x,y to row and column matrices if necessary.
    if isvector1(x) & isvector1(y) then
      [x,y] = meshgrid(x,y);
      if size(x,2)~=n & size(y,1)~=m then
        msg='The lengths of X and Y must match the size of Z.';
        return
      elseif size(x,2)~=n 
        error('The length of X must match the number of columns of Z.');
        return
      elseif size(y,1)~=m 
       msg='The length of Y must match the number of rows of Z.';
        return
      end
    elseif isvector1(x) | isvector1(y) then
      msg='X and Y must both be vectors or both be matrices.';
      return
    else
      if ~isequal(size(x),size(y),size(z)) then
        msg = 'Matrices X and Y must be the same size as Z.';
        return
      end
    end
  else // z is a vector
    if ~isvector1(x) | ~isvector1(y) then
      msg='When Z is a vector, X and Y must also be vectors.';
      return
    elseif (length(x)~=length(z) | length(y)~=length(z)) & ~((length(x)==size(z,2)) & (length(y)==size(z,1)))
      msg='X and Y must be same length as Z or the lengths of X and Y must match the size of Z.';
      return
    end
  end
end

if nargin==4 then // xyzchk(x,y,z,c)
  c = arg4;
  if ~isequal(size(z),size(c)) then 
    msg='Matrix C must be the same size as Z.';
    return;
  end
  out5 = c;
  return;
end

if nargin==5 then // xyzchk(x,y,z,xi,yi)
  xi = arg4; yi = arg5;
  if automesh(xi,yi) then
    [xi,yi] = meshgrid(xi,yi);
  elseif ~isequal(size(xi),size(yi)),
    msg='XI and YI must be the same size or vectors of different orientations.';
  end
  out5 = xi; out6 = yi;
end
endfunction 
function tf = isvector1(x)
//ISVECTOR True if x has only one non-singleton dimension.
tf = (max(size(x)) == prod(size(x)));
endfunction



